/*$6 ***************************************************************************
Copyright (C) <2007> <Sven Trenkel, Rene Weller, Gabriel Zachmann>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 ******************************************************************************* */
/**
 *  \class Settings
 *  \brief administrations class for benchmarking parameters
 *
 *  \author Sven Trenkel, Rene Weller, Gabriel Zachmann
 */
 
#ifndef SETTINGS_H
#define SETTINGS_H

/*
 * Includes ;
 */
#include <vector>

#include <OpenSG/OSGConfig.h>

#include <OpenSG/OSGBaseTypes.h>
#include <OpenSG/OSGNodePtr.h>

#include "Bench.h"

extern Algorithm    *algorithm;

/**
 *  Geometry Type that is used: planes, spheres, torus or object from a file ;
 */
enum GeometryTypeE { OBJ_PLANES, OBJ_SPHERES, OBJ_TORUS, OBJ_FILE, NUM_GEOM_TYPES };

/**
 *  Methods for configuration generation ;
 */
enum MethodTypeE { METH_SPHERE, METH_CHECKSPHERE, METH_GRID };

/**
 *  Settings ;
 */
class Settings
{

private:
    std::vector<int>                complexity;     /* set of complexities for
                                                     * simple objects like
                                                     * sphere, torus... */
    std::vector<GeometryTypeE>      geomType;       /* type of simple objects */
    std::vector<const char *>       fileNames;      /* set of objects to test */

    int                             curFile;
    std::vector<const char *>       loadNames;      /* set of precomputet
                                                     * configuration files */

    int                             curDist;
    std::vector<double>             dists;          /* set of distances */

    int                             Obj;
    std::vector<osg::NodePtr>       objects;        /* set of objects */

    int                             algo;
    std::vector<const Algorithm *>  algorithms;     /* set of algorithms */

    void                            buildObjects(); /* create the objects
                                                     * defined in geomType */
    void                            commandlineerror( char *cmd, char *parm );

public:
    Settings();
    Settings( int argc, char *argv[] );

    MethodTypeE     method;     /* method for configuration generation */
    const char      *saveName;
    const char      *qridSize;
    bool            withWindow; /* if true: benchmark starts in window mode.
                                 * Timings will be bogus */
    bool            whiteBackground;    /* true: window-background=white.
                                         * Default: background=black */
    int                             useDistObj;     /* distance object setting */

    Algorithm       *choice;
    Algorithm       *searchAlgorithm;   /* algorithm for searching for positions */
    double          dist;       /* current distance to benchmark */
    double          rotVel;     /* rotation of moving object */
    double          degVel;     /* spherical coordinate sampling with the
                                 * sphere method */
    bool            validate;   /* if true: check for false positives */
    bool            noTiming;   /* disables timing */
    bool            maxminTiming; /*Indicates wether or not we are also interested
                                    in maximum and minumum running times 
                                     default: true */

    void            parsecommandline( int argc, char *argv[] ); /* reads
                                                                 * command-line
                                                                 * arguments */

    osg::NodePtr    getObject();        /* return current tested object */
    osg::NodePtr    getMovingNode(); /* return first moving object */
    osg::NodePtr    getFixedNode(); /* return first fixed object */
    const Algorithm *getAlgorithm();    /* current benchmarked algorithm */
    const char      *getLoadname();     /* current benchmarked file */
    void            resetAlgorithms();  /* reset algorithm list */
    void            resetDists();       /* reset distance list */
    bool            nextDist();         /* return next distance in distance list */
    void            printHeader();      /* print information about benchmark */
};
#endif
