/*$6 ***************************************************************************
Copyright (C) <2007> <Sven Trenkel, Rene Weller, Gabriel Zachmann>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
 ******************************************************************************* */

/**
 *  \ Solid \
 */

/**
 *  @class Wrapper for Solid
 *  @author Sven Trenkel, Rene Weller, Gabriel Zachmann
 */

/*
 * Includes ;
 */
 
#include <stdio.h>

#include <OpenSG/OSGConfig.h>
#include <OpenSG/OSGBaseTypes.h>
#include <OpenSG/OSGNode.h>
#include <OpenSG/OSGVector.h>
#include <OpenSG/OSGMatrix.h>
#include <OpenSG/OSGGeometry.h>
#include <OpenSG/OSGGeoProperty.h>
#include <OpenSG/OSGGeoFunctions.h>
#include <OpenSG/OSGTriangleIterator.h>
#include <OpenSG/OSGBoxVolume.h>

#include <SOLID/solid.h>

#include "../Bench.h"

/*
 * Defines ;
 */
 
#define getGeom( x )    ( osg::GeometryPtr::dcast( (x)->getCore() ) )

using osg:: Vec3f;
using osg:: Pnt3f;

/*
 * Forward Declarations ;
 */
 
static void makeObject( int num, osg::NodePtr Node[] );
static int  doVCollide( int num, osg::Matrix m[] );
static void shutdown();

static tAPI API = { makeObject, doVCollide, NULL, shutdown };

static DtShapeRef  *shape;
static int         *obj;

/* -----------------------------------------------------------------------------
 *  \brief This function is called when a collision occurs
 *         Without functionality in our benchmark
 ------------------------------------------------------------------------------- */

void Callback( void                *ignore,
               DtObjectRef         obj1,
               DtObjectRef         obj2,
               const DtCollData    *dontcare )
{
}


/* -----------------------------------------------------------------------------
 *  \brief Init module and register it to the benchmarking application
 ------------------------------------------------------------------------------- */
#ifdef _WIN32
extern "C"
__declspec( dllexport )
std::vector<Algorithm *> *
initModule ( void )
#else
extern "C"
{
std::vector<Algorithm *>    *initModule( void );
}
std::vector<Algorithm *>    *
initModule ( void )
#endif
{
    std::vector<Algorithm *>    *ret = new std::vector < Algorithm * > ();
    ret->clear();
    Algorithm   *solid = new Algorithm();
    solid->shortName = "so";
    solid->name = "FreeSolid";
    solid->api = API;
    ret->push_back( solid );

    dtSetDefaultResponse( Callback, DT_SIMPLE_RESPONSE, 0 );
    return ret;
}


/* -----------------------------------------------------------------------------
 *  \brief Collision check
 *  \param m transformation matrix of moving object
 *  \param i number of transformed object (has no effect in this version)
 ------------------------------------------------------------------------------- */
static int doVCollide( int num, osg::Matrix m[] )
{
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    double  transMatrix[16] = { m[0][0][0], m[0][0][1], m[0][0][2], m[0][0][3], m[0][1][0], m[0][1][1], m[0][1][2], m[0][1][3], m[0][2][0], m[0][2][1], m[0][2][2], m[0][2][3], m[0][3][0], m[0][3][1], m[0][3][2], m[0][3][3] };
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    dtSelectObject( obj );
    dtLoadMatrixd( transMatrix );
    dtSelectObject( obj + 1 );
    dtLoadIdentity();
    return dtTest();
}

/* -----------------------------------------------------------------------------
 *  \brief Add an object to the collision detection library
 ------------------------------------------------------------------------------- */
static DtShapeRef addObject( osg::GeometryPtr geo )
{
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
    osg::TriangleIterator   ti = geo->beginTriangles(),
    ti_end = geo->endTriangles();
    Pnt3f                   Pos;
    DtShapeRef              shape = dtNewComplexShape();
    /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

    for( ; ti != ti_end; ++ti )
    {
        dtBegin( DT_SIMPLEX );
        for( int i = 0; i < 3; ++i )
        {
            Pos = ti.getPosition( i );
            dtVertex( Pos[0], Pos[1], Pos[2] );
        }

        dtEnd();
    }

    dtEndComplexShape();
    return shape;
}


/* -----------------------------------------------------------------------------
 *  \brief Make the OSG-nodes collidable
 *  \param num # of objects
 *  \param Node OSG-Nodes of the objects
 ------------------------------------------------------------------------------- */
static void makeObject( int num, osg::NodePtr Node[] )
{
    shape = new DtShapeRef[num];
    obj = new int[num + 1];
    obj[num] = -1;
    for( int i = 0; i < num; ++i )
    {
        obj[i] = i;
        shape[i] = addObject( getGeom( Node[i] ) );
        dtCreateObject( obj + i, shape[i] );
    }
}

static void shutdown()
{
    if ( !obj ) return;
    for ( int i = 0; obj[i] >= 0; ++i)
    {
        dtDeleteObject( obj + i );
        dtDeleteShape( shape[i] );
    }
    delete[] obj;
    delete[] shape;
    obj = NULL;
    shape = NULL;
}
